@@ -3,7 +3,7 @@
 // RInside.cpp: R/C++ interface class library -- Easier R embedding into C++
 //
 // Copyright (C) 2009         Dirk Eddelbuettel
-// Copyright (C) 2010 - 2015  Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2019  Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of RInside.
 //
@@ -22,7 +22,7 @@
 
 #include <RInside.h>
 #include <Callbacks.h>
-#ifndef WIN32
+#ifndef _WIN32
   #define R_INTERFACE_PTRS
   #include <Rinterface.h>
 #endif
@@ -31,10 +31,13 @@ RInside* RInside::instance_m = 0 ;
 
 const char *programName = "RInside";
 
-#ifdef WIN32
+#ifdef _WIN32
     // on Windows, we need to provide setenv which is in the file setenv.c here
     #include "setenv/setenv.c"
     extern int optind;
+
+    #include <windef.h>
+    char rHome[MAX_PATH+1];
 #endif
 
 RInside::~RInside() {           // now empty as MemBuf is internal
@@ -58,11 +61,15 @@ RInside::RInside(): global_env_m(NULL)
     initialize(0, 0, false, false, false);
 }
 
-#ifdef WIN32
+#ifdef _WIN32
+#if R_VERSION >= R_Version(4,2,0)
+static int myReadConsole(const char *prompt, unsigned char *buf, int len, int addtohistory) {
+#else
 static int myReadConsole(const char *prompt, char *buf, int len, int addtohistory) {
+#endif
     fputs(prompt, stdout);
     fflush(stdout);
-    if (fgets(buf, len, stdin))
+    if (fgets((char *)buf, len, stdin))
         return 1;
     else
         return 0;
@@ -102,7 +109,7 @@ RInside::RInside(const int argc, const char* const argv[], const bool loadRcpp,
 }
 
 // TODO: use a vector<string> would make all this a bit more readable
-void RInside::initialize(const int argc, const char* const argv[], const bool loadRcpp, 
+void RInside::initialize(const int argc, const char* const argv[], const bool loadRcpp,
                          const bool verbose, const bool interactive) {
 
     if (instance_m) {
@@ -117,7 +124,7 @@ void RInside::initialize(const int argc, const char* const argv[], const bool lo
     // generated from Makevars{.win}
     #include "RInsideEnvVars.h"
 
-    #ifdef WIN32
+    #ifdef _WIN32
     // we need a special case for Windows where users may deploy an RInside binary from CRAN
     // which will have R_HOME set at compile time to CRAN's value -- so let's try to correct
     // this here: a) allow user's setting of R_HOME and b) use R's get_R_HOME() function
@@ -125,8 +132,8 @@ void RInside::initialize(const int argc, const char* const argv[], const bool lo
         char *rhome = get_R_HOME();		// query it, including registry
         if (rhome != NULL) {                    // if something was found
             setenv("R_HOME", get_R_HOME(), 1);  // store what we got as R_HOME
-        }					// this will now be used in next blocks 
-    }                                           
+        }					// this will now be used in next blocks
+    }
     #endif
 
     for (int i = 0; R_VARS[i] != NULL; i+= 2) {
@@ -139,18 +146,19 @@ void RInside::initialize(const int argc, const char* const argv[], const bool lo
         }
     }
 
-    #ifndef WIN32
+    #ifndef _WIN32
     R_SignalHandlers = 0;               // Don't let R set up its own signal handlers
     #endif
 
     init_tempdir();
 
-    const char *R_argv[] = {(char*)programName, "--gui=none", "--no-save", 
-                            "--no-readline", "--silent", "--vanilla", "--slave"};
+    const char *R_argv[] = {(char*)programName, "--gui=none", "--no-save",
+                            "--silent", "--vanilla", "--slave", "--no-readline"};
     int R_argc = sizeof(R_argv) / sizeof(R_argv[0]);
+    if (interactive_m) R_argc--; //Deleting the --no-readline option in interactive mode
     Rf_initEmbeddedR(R_argc, (char**)R_argv);
 
-    #ifndef WIN32
+    #ifndef _WIN32
     R_CStackLimit = -1;      		// Don't do any stack checking, see R Exts, '8.1.5 Threading issues'
     #endif
 
@@ -159,8 +167,12 @@ void RInside::initialize(const int argc, const char* const argv[], const bool lo
     structRstart Rst;
     R_DefParams(&Rst);
     Rst.R_Interactive = (Rboolean) interactive_m;       // sets interactive() to eval to false
-    #ifdef WIN32
-    Rst.rhome = getenv("R_HOME");       // which is set above as part of R_VARS
+    #ifdef _WIN32
+
+    char *temp = getenv("R_HOME");       // which is set above as part of R_VARS
+    strncpy(rHome, temp, MAX_PATH);
+    Rst.rhome = rHome;
+
     Rst.home = getRUser();
     Rst.CharacterMode = LinkDLL;
     Rst.ReadConsole = myReadConsole;
@@ -173,15 +185,21 @@ void RInside::initialize(const int argc, const char* const argv[], const bool lo
     R_SetParams(&Rst);
 
     if (true || loadRcpp) {             // we always need Rcpp, so load it anyway
-        // Rf_install is used best by first assigning like this so that symbols get into the symbol table
-        // where they cannot be garbage collected; doing it on the fly does expose a minuscule risk of garbage
-        // collection -- with thanks to Doug Bates for the explanation and Luke Tierney for the heads-up
+        // Rf_install is used best by first assigning like this so that symbols get into
+        // the symbol table where they cannot be garbage collected; doing it on the fly
+        // does expose a minuscule risk of garbage collection -- with thanks to Doug Bates
+        // for the explanation and Luke Tierney for the heads-up
         SEXP suppressMessagesSymbol = Rf_install("suppressMessages");
         SEXP requireSymbol = Rf_install("require");
-        Rf_eval(Rf_lang2(suppressMessagesSymbol, Rf_lang2(requireSymbol, Rf_mkString("Rcpp"))), R_GlobalEnv);
+        SEXP reqsymlang, langobj;
+        // Protect temporaries as suggested by 'rchk', with thanks to Tomas Kalibera
+        PROTECT(reqsymlang = Rf_lang2(requireSymbol, Rf_mkString("Rcpp")));
+        PROTECT(langobj = Rf_lang2(suppressMessagesSymbol, reqsymlang));
+        Rf_eval(langobj, R_GlobalEnv);
+        UNPROTECT(2);
     }
 
-    global_env_m = new Rcpp::Environment();         // member variable for access to R's global environment 
+    global_env_m = new Rcpp::Environment();         // member variable for access to R's global environment
 
     autoloads();                        // loads all default packages, using code autogenerate from Makevars{,.win}
 
@@ -215,10 +233,10 @@ void RInside::init_tempdir(void) {
 
 void RInside::init_rand(void) { 		// code borrows from R's TimeToSeed() in datetime.c
     unsigned int pid = getpid();
-    struct timeval tv;          		// this is ifdef'ed by R, we just assume we have it 
+    struct timeval tv;          		// this is ifdef'ed by R, we just assume we have it
     gettimeofday (&tv, NULL);
     unsigned int seed = ((uint64_t) tv.tv_usec << 16) ^ tv.tv_sec;
-    seed ^= (pid << 16);         		// R 2.14.0 started to also use pid to support parallel 
+    seed ^= (pid << 16);         		// R 2.14.0 started to also use pid to support parallel
     srand(seed);
 }
 
@@ -407,6 +425,11 @@ RInside* RInside::instancePtr(){
     return instance_m;
 }
 
+void RInside::repl() {
+    R_ReplDLLinit();
+    while (R_ReplDLLdo1() > 0) {}
+}
+
 /* callbacks */
 
 #ifdef RINSIDE_CALLBACKS
@@ -472,7 +495,7 @@ void RInside_Busy( int which ){
 void RInside::set_callbacks(Callbacks* callbacks_){
     callbacks = callbacks_ ;
 
-#ifdef WIN32
+#ifdef _WIN32
     // do something to tell user that he doesn't get this
 #else
 
@@ -505,9 +528,4 @@ void RInside::set_callbacks(Callbacks* callbacks_){
 #endif
 }
 
-void RInside::repl(){
-    R_ReplDLLinit();
-    while( R_ReplDLLdo1() > 0 ) {}
-}
-
 #endif
